@@ -10,7 +10,7 @@ require 5.00404;  ##Yes, this is underhanded, but makes support for me easier
 		  ##Perl5.  01-03 weren't bug free.
 use vars (qw($VERSION $Tolerance));
 
-$VERSION = '3.0200';
+$VERSION = '3.0300';
 
 $Tolerance = 0.0;
 
@@ -18,7 +18,7 @@ package Statistics::Descriptive::Sparse;
 
 use vars qw($VERSION);
 
-$VERSION = '3.0200';
+$VERSION = '3.0300';
 
 use vars qw(%fields);
 use Carp;
@@ -76,15 +76,15 @@ sub _make_private_accessors
 ##Define the fields to be used as methods
 %fields = (
   count			=> 0,
-  mean			=> 0,
-  sum			=> 0,
-  sumsq			=> 0,
+  mean			=> undef,
+  sum			=> undef,
+  sumsq			=> undef,
   min			=> undef,
   max			=> undef,
   mindex		=> undef,
   maxdex		=> undef,
   sample_range		=> undef,
-  variance => undef,
+  variance              => undef,
   );
 
 __PACKAGE__->_make_accessors( [ grep { $_ ne "variance" } keys(%fields) ] );
@@ -176,19 +176,22 @@ sub add_data {
   $self->count($count);
   ##indicator the value is not cached.  Variance isn't commonly enough
   ##used to recompute every single data add.
-  $self->_variance(undef());
+  $self->_variance(undef);
   return 1;
 }
 
 sub standard_deviation {
   my $self = shift;  ##Myself
-  return undef if (!$self->count());
+  return if (!$self->count());
   return sqrt($self->variance());
 }
 
 ##Return variance; if needed, compute and cache it.
 sub variance {
   my $self = shift;  ##Myself
+
+  return if (!$self->count());
+  
   my $div = @_ ? 0 : 1;
   my $count = $self->count();
   if ($count < 1 + $div) {
@@ -234,7 +237,7 @@ package Statistics::Descriptive::Full;
 
 use vars qw($VERSION);
 
-$VERSION = '3.0200';
+$VERSION = '3.0300';
 
 use Carp;
 
@@ -368,22 +371,26 @@ sub sort_data {
 }
 
 sub percentile {
-  my $self = shift;
-  my $percentile = shift || 0;
-  ##Since we're returning a single value there's no real need
-  ##to cache this.
+    my $self = shift;
+    my $percentile = shift || 0;
+    ##Since we're returning a single value there's no real need
+    ##to cache this.
 
-  ##If the requested percentile is less than the "percentile bin
-  ##size" then return undef.  Check description of RFC 2330 in the
-  ##POD below.
-  my $count = $self->count();
-  return undef if $percentile < 100 / $count;
+    ##If the requested percentile is less than the "percentile bin
+    ##size" then return undef.  Check description of RFC 2330 in the
+    ##POD below.
+    my $count = $self->count();
 
-  $self->sort_data();
-  my $num = $count*$percentile/100;
-  my $index = &POSIX::ceil($num) - 1;
-  my $val = $self->_data->[$index];
-  return wantarray
+    if ((! $count) || ($percentile < 100 / $count))
+    {
+        return;  #  allow for both scalar and list context
+    }
+
+    $self->sort_data();
+    my $num = $count*$percentile/100;
+    my $index = &POSIX::ceil($num) - 1;
+    my $val = $self->_data->[$index];
+    return wantarray
     ? ($val, $index)
     : $val
     ;
@@ -411,6 +418,8 @@ sub _calc_new_median
 sub median {
     my $self = shift;
 
+    return if !$self->count;    
+    
     ##Cached?
     if (! defined($self->_median()))
     {
@@ -428,6 +437,9 @@ sub quantile {
        return;
     }
     
+    #  check data count after the args are checked - should help debugging
+    return if !$self->count;  
+    
     $self->sort_data();
 
     return $self->_data->[0] if ( $QuantileNumber == 0 );
@@ -490,6 +502,9 @@ sub trimmed_mean
         ($lower,$upper) = ($_[0],$_[1]);
     }
 
+    #  check data count after the args
+    return if !$self->count;    
+
     ##Cache
     my $thistm = join ':',$lower,$upper;
     my $cache = $self->_trimmed_mean_cache();
@@ -553,40 +568,36 @@ sub mode
     if (!defined ($self->_mode()))
     {
         my $mode = 0;
-        my $occurances= 0;
-        my $flag = 1;
+        my $occurances = 0;
 
         my %count;
 
         foreach my $item (@{ $self->_data() })
         {
-            $count{$item}++;
-            $flag = 0 if ($count{$item} > 1);
-        }
-
-        #Distribution is flat - no mode exists
-        if ($flag)
-        {
-            return undef;
-        }
-
-        foreach my $val (keys %count)
-        {
-            if ($count{$val} > $occurances)
+            my $count = ++$count{$item};
+            if ($count > $occurances)
             {
-                $occurances = $count{$val};
-                $mode = $val;
+                $mode = $item;
+                $occurances = $count;
             }
         }
 
-        $self->_mode($mode);
+        $self->_mode(
+            ($occurances > 1)
+            ? {exists => 1, mode => $mode}
+            : {exists => 0,}
+        );
     }
 
-    return $self->_mode();
+    my $m = $self->_mode;
+
+    return $m->{'exists'} ? $m->{mode} : undef;
 }
 
 sub geometric_mean {
     my $self = shift;
+    
+    return if !$self->count;
 
     if (!defined($self->_geometric_mean()))
     {
@@ -1016,7 +1027,8 @@ for which F(x) >= a given percentage.  So the 50th percentile of the
 example above is 4, since F(4) = 3/6 = 50%; the 25th percentile is
 -2, since F(-5) = 1/6 < 25%, and F(-2) = 2/6 >= 25%; the 100th
 percentile is 18; and the 0th percentile is -infinity, as is the 15th
-percentile.
+percentile, which for ease of handling and backward compatibility is returned
+as undef() by the function.
 
 Care must be taken when using percentiles to summarize a sample,
 because they can lend an unwarranted appearance of more precision
@@ -1078,9 +1090,11 @@ it will return undef for both of those cases.
 
 Returns the geometric mean of the data.
 
-=item $stat->mode();
+=item my $mode = $stat->mode();
 
-Returns the mode of the data. 
+Returns the mode of the data. The mode is the most commonly occuring datum.
+See L<http://en.wikipedia.org/wiki/Mode_%28statistics%29> . If all values
+occur only once, then mode() will return undef. 
 
 =item $stat->trimmed_mean(ltrim[,utrim]);
 
